/* _MiscTreeBrowserNode
 *
 */

// IMPORTS

#import <libc.h>
#import <ansi/assert.h>

#import <objc/List.h>
#import <objc/hashtable.h>

#import "_MiscTreeBrowserNode.h"


// IMPLEMENTATION

@implementation _MiscTreeBrowserNode

- initWithString:(const char *)aString parent:anObject
{
	[super init];

	[self setStringValue:aString];
	[self setParent:anObject];

	return self;
}

- init
{
	return [self initWithString:NULL parent:nil];
}

- free
{
	if (stringValue) free(stringValue);
	stringValue = NULL;

	children = [[children freeObjects] free];

	return [super free];
}

- (BOOL)canFree
{
	return YES;
}

- setParent:(id<MiscTreeBrowserNodes>)aNode
{
	assert(aNode != self);
	assert((aNode == nil) || (parent == nil));

	if (parent != aNode)
	{
		parent = aNode;
		[parent addChild:self];
	}

	return self;
}

// MiscTreeBrowserNodes methods

- (const char *)stringValue
{
	return stringValue;
}

- parent
{
	return parent;
}

- (unsigned int)childCount
{
	return children ? [children count] : 0;
}

- childAt:(int)index
{
	return children ? [children objectAt:index] : nil;
}

- (int)indexOfChild:(id<MiscTreeBrowserNodes>)aNode
{
	return children ? [children indexOf:aNode] : NX_NOT_IN_LIST;
}

// MiscTreeBrowserEditing methods

- setStringValue:(const char *)aString
{
	if (stringValue)
	{
		free(stringValue);
	}
	stringValue = aString ? NXCopyStringBuffer(aString) : NULL;

	return self;
}

// MiscTreeBrowserModifications methods

- (BOOL)canAddChild:(id<MiscTreeBrowserNodes>)aNode
			  after:(id<MiscTreeBrowserNodes>)sibling
{
	if (sibling)
	{
		int sibIndex = [self indexOfChild:sibling];
		int nodeIndex = [self indexOfChild:aNode];

		if ((sibIndex == NX_NOT_IN_LIST) ||
			((nodeIndex != NX_NOT_IN_LIST) && (sibIndex == (nodeIndex - 1))))
		{
			return NO;
		}
	}

	return YES;
}

- addChild:(id<MiscTreeBrowserNodes>)aNode
	 after:(id<MiscTreeBrowserNodes>)sibling
{
	id pa = [aNode parent];

	assert(aNode != self);
	assert((pa == self) || (pa == nil));

	if (children)
	{
		int index;

		if (sibling == nil)
		{						// insert as first child
			[children removeObject:aNode];
			[children insertObject:aNode at:0];
		}
		else if ([children indexOf:sibling] != NX_NOT_IN_LIST)
		{
			[children removeObject:aNode];
			index = [children indexOf:sibling];
			[children insertObject:aNode at:(index + 1)];
		}
	}
	else if (sibling == nil)
	{							// first child
		children = [[List alloc] init];
		[children addObject:aNode];
	}
	else
	{							// do nothing
		return nil;
	}

	if (pa != self)
	{
		[(id)aNode setParent:self];
	}

	return self;
}

- (BOOL)canAddChild:(id<MiscTreeBrowserNodes>)aNode
{
	return YES;
}

- addChild:(id<MiscTreeBrowserNodes>)aNode
{
	id pa = [aNode parent];

	assert((pa == self) || (pa == nil));
	if ([self indexOfChild:aNode] == NX_NOT_IN_LIST)
	{
		return [self addChild:aNode after:[children lastObject]];
	}
	else
	{
		return self;
	}
}

- (BOOL)canRemoveChild:(id<MiscTreeBrowserNodes>)aNode
{
	return YES;
}

- removeChild:(id<MiscTreeBrowserNodes>)aNode
{
	assert([aNode parent] == self);

	if (children)
	{
		unsigned int index = [children indexOf:aNode];

		if (index != NX_NOT_IN_LIST)
		{
			[(id)aNode setParent:nil];

			return [children removeObjectAt:index];
		}
	}

	return nil;
}

@end
