//-----------------------------------------------------------------------------
//
//		Copyright (C) 1997 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// $Id: MiscAssert.cc,v 1.2 97/07/01 10:33:36 sunshine Exp $
// $Log:		MiscAssert.cc,v $
// Revision 1.2	 97/07/01  10:33:36	 sunshine
// v128: Re-engineered to properly raise and report assertion failure.
// Formatted error message is now stored in NXHandler's data1 so exception
// catchers can make use of it.	 Published and documented exception code.
// 
// Revision 1.1	 97/06/10  05:21:41	 sunshine
// v127: Assertion facility which raises exception rather than calling abort()
//-----------------------------------------------------------------------------
#ifdef __GNUC__
# pragma implementation
#endif
#import "MiscAssert.h"
#import <misckit/MiscTableTypes.h>
#import "bool.h"
extern "Objective-C" {
#import <appkit/errors.h>
#import <appkit/nextstd.h>
}

//-----------------------------------------------------------------------------
// assert_reporter
//-----------------------------------------------------------------------------
static void assert_reporter( NXHandler const* err )
	{
	if (err->code == MISC_TABLE_SCROLL_ASSERTION)
		NXLogError( (char const*)err->data1 );
	else
		NXLogError( "MiscTableScroll: unrecognized error code %d", err->code );
	}


//-----------------------------------------------------------------------------
// assert_init
//-----------------------------------------------------------------------------
static inline void assert_init()
	{
	static bool done = false;
	if (!done)
		{
		done = true;
		NXRegisterErrorReporter( MISC_TABLE_SCROLL_ERROR_BASE,
								 MISC_TABLE_SCROLL_ERROR_MAX,
								 assert_reporter );
		}
	}


//-----------------------------------------------------------------------------
// MiscTableScrollAssert
//-----------------------------------------------------------------------------
void MiscTableScrollAssert( char const* expr, char const* file, int line )
	{
	assert_init();
	static char const ASSERT_FMT[] = "%s:%u: failed assertion `%s'";
	int const len = sizeof( ASSERT_FMT ) + strlen( expr ) + strlen( file ) + 32;
	char* s; NXAllocErrorData( len, (void**)&s );
	sprintf( s, ASSERT_FMT, file, line, expr );
	NX_RAISE( MISC_TABLE_SCROLL_ASSERTION, s, 0 );
	}
