//=============================================================================
//
//		Copyright (C) 1995,1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscColorList.cc
//
//		An extensible array of colors that can act as a color palette to
//		reduce the storage overhead of storing a lot of colors.	 Also
//		useful for speeding color comparisons in some cases.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscColorList.cc,v 1.1 96/08/30 14:51:54 sunshine Exp $
// $Log:		MiscColorList.cc,v $
// Revision 1.1	 96/08/30  14:51:54	 sunshine
// Extensible array of colors acting as a color palette.
// 
//-----------------------------------------------------------------------------
#ifdef __GNUC__
#pragma implementation
#endif
#include "MiscColorList.h"

extern "C" {
#include <stdlib.h>		// free(), malloc(), realloc()
}


//-----------------------------------------------------------------------------
// Destructor
//-----------------------------------------------------------------------------
MiscColorList::~MiscColorList()
	{
	free( colors );
	}


//-----------------------------------------------------------------------------
// Constructor
//-----------------------------------------------------------------------------
MiscColorList::MiscColorList()
	{
	num_colors = 0;
	max_colors = 8;
	colors = (NXColor*) malloc( max_colors * sizeof(*colors) );
	}


//-----------------------------------------------------------------------------
// store
//-----------------------------------------------------------------------------
int MiscColorList::store( NXColor c )
	{
	for (int i = 0; i < num_colors; i++)
		if (NXEqualColor( colors[i], c ))
			return i;

	if (num_colors >= max_colors)
		{
		max_colors += max_colors;
		colors = (NXColor*) realloc( colors, max_colors * sizeof(*colors) );
		}

	colors[ num_colors++ ] = c;
	return num_colors - 1;
	}
