#ifndef __MiscExporter_h
#define __MiscExporter_h
//=============================================================================
//
//		Copyright (C) 1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscExporter.h
//
//		Object that exports the contents of an MiscTableScroll in
//		various useful formats.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscExporter.h,v 1.2 96/09/25 17:09:06 zarnuk Exp $
// $Log:		MiscExporter.h,v $
//	Revision 1.2  96/09/25	17:09:06  zarnuk
//	Added commonInstance.  exportTableScroll:(toFilename:) methods now return
//	an integer success/fail code.
//	
//	Revision 1.1  96/09/25	11:53:59  zarnuk
//	Exports the contents of table scroll in various formats.
//	
//-----------------------------------------------------------------------------
#import <misckit/MiscTableTypes.h>
MISC_TS_EXTERN_BEGIN( "Objective-C" )
#import <objc/Object.h>
MISC_TS_EXTERN_END

@class MiscTableScroll;

typedef enum
		{
		MISC_EXPORT_ASCII_FIXED,
		MISC_EXPORT_ASCII_TAB,
		MISC_EXPORT_ASCII_DELIMITED,
		MISC_EXPORT_DBF
		} MiscExportFormat;

#define MISC_EXPORT_FORMAT_FIRST		MISC_EXPORT_ASCII_FIXED
#define MISC_EXPORT_FORMAT_LAST			MISC_EXPORT_DBF
#define MISC_EXPORT_FORMAT_DEFAULT		MISC_EXPORT_ASCII_FIXED


typedef enum							// For ASCII formats only.
		{
		MISC_EXPORT_TITLES_OFF,			// No titles.
		MISC_EXPORT_TITLES_ON,			// Full width, expand column if needed.
		MISC_EXPORT_TITLES_WRAP,		// Wrap titles within data width.
		MISC_EXPORT_TITLES_TRUNCATE,	// Truncate titles to data width.
		} MiscExportTitleMode;

#define MISC_EXPORT_TITLES_FIRST		MISC_EXPORT_TITLES_OFF
#define MISC_EXPORT_TITLES_LAST			MISC_EXPORT_TITLES_TRUNCATE
#define MISC_EXPORT_TITLES_ROW_DEFAULT	MISC_EXPORT_TITLES_OFF
#define MISC_EXPORT_TITLES_COL_DEFAULT	MISC_EXPORT_TITLES_ON


typedef enum
		{
		MISC_EXPORT_GRID_OFF,			// No grid
		MISC_EXPORT_GRID_LINE,			// Lines, col-grid='|', row-grid='-'
		MISC_EXPORT_GRID_SPACE,			// Spaces col-grid=' ', row-grid=' '
		} MiscExportGridMode;

#define MISC_EXPORT_GRID_FIRST			MISC_EXPORT_GRID_OFF
#define MISC_EXPORT_GRID_LAST			MISC_EXPORT_GRID_SPACE
#define MISC_EXPORT_GRID_ROW_DEFAULT	MISC_EXPORT_GRID_OFF
#define MISC_EXPORT_GRID_COL_DEFAULT	MISC_EXPORT_GRID_LINE


@interface MiscExporter : Object
		{
		MiscTableScroll*		tableScroll;	// Defaults
		MiscExportFormat		exportFormat;	// MISC_EXPORT_ASCII_FIXED
		MiscExportTitleMode		rowTitleMode;	// MISC_EXPORT_TITLES_OFF
		MiscExportTitleMode		colTitleMode;	// MISC_EXPORT_TITLES_ON
		MiscExportGridMode		rowGrid;		// MISC_EXPORT_GRID_OFF
		MiscExportGridMode		colGrid;		// MISC_EXPORT_GRID_LINE
		}

- init;

- (int) exportTableScroll:(MiscTableScroll*)ts; // Run SavePanel.
- (int) exportTableScroll:(MiscTableScroll*)ts toFilename:(char const*) name;

- (void) setExportFormat:(MiscExportFormat)exportFormat;
- (void) setRowExportTitleMode:(MiscExportTitleMode)rowTitleMode;
- (void) setColExportTitleMode:(MiscExportTitleMode)colTitleMode;
- (void) setRowExportGridMode:(MiscExportGridMode)rowExportGridMode;
- (void) setColExportGridMode:(MiscExportGridMode)colExportGridMode;

- (MiscExportFormat) getExportFormat;
- (MiscExportTitleMode) getRowExportTitleMode;
- (MiscExportTitleMode) getColExportTitleMode;
- (MiscExportGridMode) getRowExportGridMode;
- (MiscExportGridMode) getColExportGridMode;

+ (MiscExporter*) commonInstance;
@end

#endif // __MiscExporter_h
