#ifndef __MiscTableScroll_h
#define __MiscTableScroll_h
//=============================================================================
//
//		Copyright (C) 1995-1997 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// <misckit/MiscTableScroll.h>
//
//		ScrollView class that displays a 2-D table of cells.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableScroll.h,v 1.22 97/02/11 19:29:46 sunshine Exp $
// $Log:		MiscTableScroll.h,v $
// Revision 1.22  97/02/11	19:29:46  sunshine
// v123: Added new ...:byExtension: variations to all selection methods.
// Added deselection methods.  Published -selectedCell.
// reflectSelection --> selectionChanged
// 
// Revision 1.21  97/02/05	08:06:13  sunshine
// v122: Added -tracking to fix lazy-mode mouse-tracking display problem.
// Added explicit (id) for return and arguments.  Unified whitespace.
// 
// Revision 1.20  96/12/30	16:29:40  zarnuk
// v113: Migrated cell editing to here from table view.	 Added methods to
// programmatically set the size of the titles.
//-----------------------------------------------------------------------------
#import <misckit/MiscTableTypes.h>

MISC_TS_EXTERN_BEGIN( "Objective-C" )
#import <appkit/ScrollView.h>
MISC_TS_EXTERN_END

MISC_TS_CLASS_DEF( MiscTableBorder );
MISC_TS_CLASS_DEF( MiscDelegateFlags );
@class MiscTableScroll, MiscTableView, MiscBorderView, MiscNullView;
@class MiscIntList;
@class ClipView, Font, Storage, Text;


typedef struct MiscCellEditInfo
		{
		BOOL					editing;		// Is editing in progress?
		MiscCoord_P				row;			// Coords of edited cell.
		MiscCoord_P				col;
		Text*					editor;			// Field editor.
		id						cell;			// Temporary editing cell.
		} MiscCellEditInfo;


typedef struct MiscBorderInfo
		{
		MiscTableBorder*		border;
		MiscBorderView*			view;
		ClipView*				clip;
		int						sort_vector_len;
		int*					sort_vector;
		BOOL					isOn;
		BOOL					autoSort;
		BOOL					constrain_min;
		BOOL					constrain_max;
		} MiscBorderInfo;


@interface MiscTableScroll : ScrollView
	{
@private
	MiscTableView*		tableView;
	MiscBorderInfo*		info[2];		// { &colInfo, &rowInfo }
	MiscBorderInfo		colInfo;
	MiscBorderInfo		rowInfo;
	MiscNullView*		nullView;
	Font*				font;
	NXColor				textColor;
	NXColor				backgroundColor;
	NXColor				highlightTextColor;
	NXColor				highlightBackgroundColor;
	id					nextText;
	id					previousText;
	id					delegate;
	id					dataDelegate;
	MiscDelegateFlags*	delegateFlags;
	MiscDelegateFlags*	dataDelegateFlags;
	id					target;
	id					doubleTarget;
	SEL					action;
	SEL					doubleAction;
	int					tag;
	MiscCompareEntryFunc sort_entry_func;
	MiscCompareSlotFunc sort_slot_func;
	BOOL				enabled;
	BOOL				lazy;
	MiscSelectionMode	mode;
	int					num_cols;		// Currently active number of
	int					num_rows;		// columns and rows.
	int					max_rows;		// Highwater mark for Cell allocations.
	int					max_cells;
	id*					cells;
	id					pageHeader;
	id					pageFooter;
	MiscCellEditInfo	editInfo;
	BOOL				tracking;
	}

- (id)initFrame:(NXRect const*)frameRect;
- (id)free;


// DELEGATE -------------------------------------------------------------------
- (id)delegate;
- (id)setDelegate:(id)obj;
- (id)dataDelegate;
- (id)setDataDelegate:(id)obj;


// TARGET / ACTION ------------------------------------------------------------
- (id)target;
- (id)setTarget:(id)obj;
- (id)doubleTarget;
- (id)setDoubleTarget:(id)obj;
- (SEL)action;
- (id)setAction:(SEL)new_sel;
- (SEL)doubleAction;
- (id)setDoubleAction:(SEL)new_sel;

- (id)sendAction:(SEL)theAction to:(id)theTarget;
- (id)sendAction;
- (id)sendActionIfEnabled;
- (id)sendDoubleAction;
- (id)sendDoubleActionIfEnabled;

- (int)tag;
- (id)setTag:(int)x;

- (BOOL)tracking;
- (MiscCoord_P)borderClickedSlot:(MiscBorderType)b;
- (MiscCoord_P)clickedRow;
- (MiscCoord_P)clickedCol;
- (id)clickedCell;


// NEXT-TEXT CHAIN ------------------------------------------------------------
- (id)nextText;
- (id)previousText;
- (id)setNextText:(id)obj;
- (id)setPreviousText:(id)obj;
- (id)selectText:(id)sender;
- (BOOL)isEnabled;
- (id)setEnabled:(BOOL)flag;
- (BOOL)isSelectable;


// FONT -----------------------------------------------------------------------
- (id)font;
- (id)setFont:(id)newFont;
- (id)changeFont:(id)sender;
+ (Font*)defaultFont;


// COLOR ----------------------------------------------------------------------
- (NXColor)backgroundColor;
- (id)setBackgroundColor:(NXColor)value;		// Sent to all cells.
- (NXColor)textColor;
- (id)setTextColor:(NXColor)value;				// Sent to all cells.
- (NXColor)highlightBackgroundColor;
- (id)setHighlightBackgroundColor:(NXColor)val; //Send to all cells.
- (NXColor)highlightTextColor;
- (id)setHighlightTextColor:(NXColor)value;		// Sent to all cells.

- (float)backgroundGray;						// Gray values are converted
- (id)setBackgroundGray:(float)value;			// to/from color values.
- (float)textGray;
- (id)setTextGray:(float)value;
- (float)highlightBackgroundGray;
- (id)setHighlightBackgroundGray:(float)value;
- (float)highlightTextGray;
- (id)setHighlightTextGray:(float)value;

- (id)setColor:(NXColor)value;			// Alias for -setBackgroundColor:
- (NXColor)color;						// Alias for -backgroundColor

+ (NXColor)defaultBackgroundColor;
+ (NXColor)defaultTextColor;
+ (NXColor)defaultHighlightBackgroundColor;
+ (NXColor)defaultHighlightTextColor;


// SLOT ORDER -----------------------------------------------------------------
- (id)border:(MiscBorderType)b slotOrder:(MiscIntList*)list;
- (id)colOrder:(MiscIntList*)list;
- (id)rowOrder:(MiscIntList*)list;

- (id)border:(MiscBorderType)b setSlotOrder:(MiscIntList*)list;
- (id)setColOrder:(MiscIntList*)list;
- (id)setRowOrder:(MiscIntList*)list;

- (char*)border:(MiscBorderType)b slotOrderAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;
- (char*)colOrderAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;
- (char*)rowOrderAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;

- (id)border:(MiscBorderType)b setSlotOrderFromString:(char const*)s;
- (id)setColOrderFromString:(char const*)s;
- (id)setRowOrderFromString:(char const*)s;


// SLOT SIZES -----------------------------------------------------------------
- (id)border:(MiscBorderType)b slotSizes:(MiscIntList*)list;
- (id)colSizes:(MiscIntList*)list;
- (id)rowSizes:(MiscIntList*)list;

- (id)border:(MiscBorderType)b setSlotSizes:(MiscIntList*)list;
- (id)setColSizes:(MiscIntList*)list;
- (id)setRowSizes:(MiscIntList*)list;

- (char*)border:(MiscBorderType)b slotSizesAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;
- (char*)colSizesAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;
- (char*)rowSizesAsString:(char*)buff
		size:(int)buff_size canExpand:(BOOL)canExpand;

- (id)border:(MiscBorderType)b setSlotSizesFromString:(char const*)s;
- (id)setColSizesFromString:(char const*)s;
- (id)setRowSizesFromString:(char const*)s;


// MULTICAST MESSAGES ---------------------------------------------------------
- (id)sendAction:(SEL)aSelector to:(id)anObject forAllCells:(BOOL)flag;

- (int)makeCellsPerform:(SEL)aSel selectedOnly:(BOOL)flag;
- (int)makeCellsPerform:(SEL)aSel with:(id)arg1 selectedOnly:(BOOL)flag;
- (int)makeCellsPerform:(SEL)aSel with:(id)arg1 with:(id)arg2
		selectedOnly:(BOOL)flag;

- (int)makeCellsPerform:(SEL)aSel;
- (int)makeCellsPerform:(SEL)aSel with:(id)arg1;
- (int)makeCellsPerform:(SEL)aSel with:(id)arg1 with:(id)arg2;


// FINDING CELLS --------------------------------------------------------------
- (int)border:(MiscBorderType)b findSlotWithTag:(int)x;
- (int)findColWithTag:(int)x;
- (int)findRowWithTag:(int)x;
- (id)findCellWithTag:(int)x;
- (id)findCellWithTag:(int)x row:(int*)row col:(int*)col;
- (id)findCell:(id)cell row:(int*)row col:(int*)col;


// SELECTION ------------------------------------------------------------------
- (MiscSelectionMode)selectionMode;
- (void)setSelectionMode:(MiscSelectionMode)x;

- (BOOL)borderHasSelection:(MiscBorderType)b;
- (BOOL)hasRowSelection;
- (BOOL)hasColSelection;
- (BOOL)borderHasMultipleSelection:(MiscBorderType)b;
- (BOOL)hasMultipleRowSelection;
- (BOOL)hasMultipleColSelection;
- (unsigned int)borderNumSelectedSlots:(MiscBorderType)b;
- (unsigned int)numSelectedRows;
- (unsigned int)numSelectedCols;
- (BOOL)border:(MiscBorderType)b slotIsSelected:(MiscCoord_P)slot;
- (BOOL)rowIsSelected:(MiscCoord_P)row;
- (BOOL)colIsSelected:(MiscCoord_P)col;
- (BOOL)cellIsSelected:(MiscCoord_P)row :(MiscCoord_P)col;

- (MiscCoord_P)borderSelectedSlot:(MiscBorderType)b;
- (MiscCoord_P)selectedRow;
- (MiscCoord_P)selectedCol;
- (id)selectedCell;
- (void)border:(MiscBorderType)b selectedTags:(MiscIntList*)tags;
- (void)selectedRowTags:(MiscIntList*)tags;
- (void)selectedColTags:(MiscIntList*)tags;
- (void)border:(MiscBorderType)b selectedSlots:(MiscIntList*)slots;
- (void)selectedRows:(MiscIntList*)rows;
- (void)selectedCols:(MiscIntList*)cols;

- (void)border:(MiscBorderType)b selectSlot:(MiscCoord_P)slot
		byExtension:(BOOL)flag;
- (void)border:(MiscBorderType)b selectSlot:(MiscCoord_P)slot;
- (void)selectRow:(MiscCoord_P)row byExtension:(BOOL)flag;
- (void)selectRow:(MiscCoord_P)row;
- (void)selectCol:(MiscCoord_P)col byExtension:(BOOL)flag;
- (void)selectCol:(MiscCoord_P)col;
- (void)border:(MiscBorderType)b selectTags:(MiscIntList*)tags
		 byExtension:(BOOL)flag;
- (void)border:(MiscBorderType)b selectTags:(MiscIntList*)tags;
- (void)selectRowTags:(MiscIntList*)tags byExtension:(BOOL)flag;
- (void)selectRowTags:(MiscIntList*)tags;
- (void)selectColTags:(MiscIntList*)tags byExtension:(BOOL)flag;
- (void)selectColTags:(MiscIntList*)tags;
- (void)border:(MiscBorderType)b selectSlots:(MiscIntList*)slots
		 byExtension:(BOOL)flag;
- (void)border:(MiscBorderType)b selectSlots:(MiscIntList*)slots;
- (void)selectRows:(MiscIntList*)rows byExtension:(BOOL)flag;
- (void)selectRows:(MiscIntList*)rows;
- (void)selectCols:(MiscIntList*)cols byExtension:(BOOL)flag;
- (void)selectCols:(MiscIntList*)cols;
- (void)borderSelectAll:(MiscBorderType)b;
- (void)selectAllRows;
- (void)selectAllCols;
- (id)selectAll:(id)sender;		// -selectAllRows and sends action to target.

- (void)border:(MiscBorderType)b deselectSlot:(MiscCoord_P)slot;
- (void)deselectRow:(MiscCoord_P)row;
- (void)deselectCol:(MiscCoord_P)col;
- (void)border:(MiscBorderType)b deselectTags:(MiscIntList*)tags;
- (void)deselectRowTags:(MiscIntList*)tags;
- (void)deselectColTags:(MiscIntList*)tags;
- (void)border:(MiscBorderType)b deselectSlots:(MiscIntList*)slots;
- (void)deselectRows:(MiscIntList*)rows;
- (void)deselectCols:(MiscIntList*)cols;
- (void)borderClearSelection:(MiscBorderType)b;
- (void)clearRowSelection;
- (void)clearColSelection;
- (void)clearSelection;
- (id)deselectAll:(id)sender;	// -clearSelection and sends action to target.


// MOUSE & KEYBOARD TRACKING (SELECTION ORIENTATION) --------------------------
- (void)trackBy:(MiscBorderType)b;
- (MiscBorderType)trackingBy;


// KEYBOARD CURSOR ------------------------------------------------------------
- (MiscCoord_P)borderCursor:(MiscBorderType)b;
- (MiscCoord_P)cursorRow;
- (MiscCoord_P)cursorCol;
- (void)border:(MiscBorderType)b setCursor:(MiscCoord_P)slot;
- (void)setCursorRow:(MiscCoord_P)row;
- (void)setCursorCol:(MiscCoord_P)col;
- (void)borderClearCursor:(MiscBorderType)b;
- (void)clearCursorRow;
- (void)clearCursorCol;
- (void)clearCursor;
- (BOOL)borderHasValidCursor:(MiscBorderType)b;
- (BOOL)hasValidCursorRow;
- (BOOL)hasValidCursorCol;
- (void)disableCursor;			// Can nest.
- (void)reenableCursor;
- (BOOL)isCursorEnabled;


// GLOBAL SIZE LIMITS ---------------------------------------------------------
- (NXCoord)totalSize:(MiscBorderType)b;
- (NXCoord)totalWidth;
- (NXCoord)totalHeight;

- (NXCoord)minTotalSize:(MiscBorderType)b;
- (NXCoord)maxTotalSize:(MiscBorderType)b;
- (NXCoord)minTotalWidth;
- (NXCoord)maxTotalWidth;
- (NXCoord)minTotalHeight;
- (NXCoord)maxTotalHeight;

- (void)border:(MiscBorderType)b setMinTotalSize:(NXCoord)size;
- (void)border:(MiscBorderType)b setMaxTotalSize:(NXCoord)size;
- (void)setMinTotalWidth:(NXCoord)size;
- (void)setMaxTotalWidth:(NXCoord)size;
- (void)setMinTotalHeight:(NXCoord)size;
- (void)setMaxTotalHeight:(NXCoord)size;

- (void)constrainSize;			// Applies constraints, updates views.

- (BOOL)minTotalSizeIsConstrained:(MiscBorderType)b;
- (BOOL)maxTotalSizeIsConstrained:(MiscBorderType)b;
- (BOOL)minTotalWidthIsConstrained;
- (BOOL)maxTotalWidthIsConstrained;
- (BOOL)minTotalHeightIsConstrained;
- (BOOL)maxTotalHeightIsConstrained;

- (void)border:(MiscBorderType)b constrainMinTotalSize:(BOOL)flag;
- (void)border:(MiscBorderType)b constrainMaxTotalSize:(BOOL)flag;
- (void)constrainMinTotalWidth:(BOOL)flag;
- (void)constrainMaxTotalWidth:(BOOL)flag;
- (void)constrainMinTotalHeight:(BOOL)flag;
- (void)constrainMaxTotalHeight:(BOOL)flag;


// SLOTS ----------------------------------------------------------------------
- (MiscTableBorder*)border:(MiscBorderType)b;
- (BOOL)borderTitlesOn:(MiscBorderType)b;
- (BOOL)border:(MiscBorderType)b setTitlesOn:(BOOL)on_off;
		// -border:setTitlesOn: Returns YES if changed.
- (MiscTableTitleMode)borderTitleMode:(MiscBorderType)b;
- (void)border:(MiscBorderType)b setTitleMode:(MiscTableTitleMode)x;
- (NXCoord)borderTitlesSize:(MiscBorderType)b;
- (void)border:(MiscBorderType)b setTitlesSize:(NXCoord)x;

- (id)border:(MiscBorderType)b moveSlotFrom:(int)pos to:(int)pos;
- (int)border:(MiscBorderType)b slotPosition:(int)slot;
- (int)border:(MiscBorderType)b slotAtPosition:(int)pos;
- (void)border:(MiscBorderType)b physicalToVisual:(MiscIntList*)list;
- (void)border:(MiscBorderType)b visualToPhysical:(MiscIntList*)list;

- (BOOL)sizeableSlots:(MiscBorderType)b;
- (BOOL)draggableSlots:(MiscBorderType)b;
- (BOOL)modifierDragSlots:(MiscBorderType)b;
- (NXCoord)uniformSizeSlots:(MiscBorderType)b;

- (NXCoord)border:(MiscBorderType)b slotAdjustedSize:(int)slot;
- (NXCoord)border:(MiscBorderType)b slotSize:(int)slot;
- (NXCoord)border:(MiscBorderType)b slotMinSize:(int)slot;
- (NXCoord)border:(MiscBorderType)b slotMaxSize:(int)slot;
- (NXCoord)border:(MiscBorderType)b slotDataSize:(int)slot;
- (BOOL)border:(MiscBorderType)b slotIsSizeable:(int)slot;
- (BOOL)border:(MiscBorderType)b slotExpandsToData:(int)slot;
- (BOOL)border:(MiscBorderType)b slotIsAutosize:(int)slot;
- (char const*)border:(MiscBorderType)b slotTitle:(int)slot;
- (int)border:(MiscBorderType)b slotTag:(int)slot;
- (MiscTableCellStyle)border:(MiscBorderType)b slotCellType:(int)slot;
- (id)border:(MiscBorderType)b slotCellPrototype:(int)slot;

- (void)border:(MiscBorderType)b setSizeableSlots:(BOOL)flag;
- (void)border:(MiscBorderType)b setDraggableSlots:(BOOL)flag;
- (void)border:(MiscBorderType)b setModifierDragSlots:(BOOL)flag;
- (void)border:(MiscBorderType)b setUniformSizeSlots:(NXCoord)uniform_size;

- (void)border:(MiscBorderType)b setSlot:(int)n size:(NXCoord)size;
- (void)border:(MiscBorderType)b setSlot:(int)n minSize:(NXCoord)size;
- (void)border:(MiscBorderType)b setSlot:(int)n maxSize:(NXCoord)size;
- (void)border:(MiscBorderType)b setSlot:(int)n dataSize:(NXCoord)size;
- (void)border:(MiscBorderType)b setSlot:(int)n sizeable:(BOOL)flag;
- (void)border:(MiscBorderType)b setSlot:(int)n expandsToData:(BOOL)flag;
- (void)border:(MiscBorderType)b setSlot:(int)n autosize:(BOOL)flag;
- (void)border:(MiscBorderType)b setSlot:(int)n title:(char const*)title;
- (void)border:(MiscBorderType)b setSlot:(int)n tag:(int)tag;
- (void)border:(MiscBorderType)b setSlot:(int)n
				cellType:(MiscTableCellStyle)t;
- (void)border:(MiscBorderType)b setSlot:(int)n cellPrototype:(id)cell;

// COLS -----------------------------------------------------------------------
- (MiscTableBorder*)colBorder;
- (BOOL)colTitlesOn;
- (BOOL)setColTitlesOn:(BOOL)on_off;	// Returns YES if changed.
- (MiscTableTitleMode)colTitleMode;
- (void)setColTitleMode:(MiscTableTitleMode)x;
- (NXCoord)colTitlesHeight;
- (void)setColTitlesHeight:(NXCoord)x;

- (id)moveColFrom:(int)pos to:(int)pos;
- (int)colPosition:(int)col;
- (int)colAtPosition:(int)pos;

- (NXCoord)uniformSizeCols;
- (BOOL)sizeableCols;
- (BOOL)draggableCols;
- (BOOL)modifierDragCols;
- (NXCoord)colAdjustedSize:(int)col;
- (NXCoord)colSize:(int)col;
- (NXCoord)colMinSize:(int)col;
- (NXCoord)colMaxSize:(int)col;
- (NXCoord)colDataSize:(int)col;
- (BOOL)colIsSizeable:(int)col;
- (BOOL)colExpandsToData:(int)col;
- (BOOL)colIsAutosize:(int)col;
- (char const*)colTitle:(int)col;
- (int)colTag:(int)col;
- (MiscTableCellStyle)colCellType:(int)col;
- (id)colCellPrototype:(int)col;

- (void)setSizeableCols:(BOOL)flag;
- (void)setDraggableCols:(BOOL)flag;
- (void)setModifierDragCols:(BOOL)flag;
- (void)setUniformSizeCols:(NXCoord)uniform_size;

- (void)setCol:(int)col size:(NXCoord)size;
- (void)setCol:(int)col minSize:(NXCoord)size;
- (void)setCol:(int)col maxSize:(NXCoord)size;
- (void)setCol:(int)col dataSize:(NXCoord)size;
- (void)setCol:(int)col sizeable:(BOOL)flag;
- (void)setCol:(int)col expandsToData:(BOOL)flag;
- (void)setCol:(int)col autosize:(BOOL)flag;
- (void)setCol:(int)col title:(char const*)title;
- (void)setCol:(int)col tag:(int)tag;
- (void)setCol:(int)col cellType:(MiscTableCellStyle)type;
- (void)setCol:(int)col cellPrototype:(id)cell;

- (int)numVisibleCols;
- (int)firstVisibleCol;
- (int)lastVisibleCol;
- (BOOL)colIsVisible:(int)n;
- (void)setFirstVisibleCol:(int)n;
- (void)setLastVisibleCol:(int)n;


// ROWS -----------------------------------------------------------------------
- (MiscTableBorder*)rowBorder;
- (BOOL)rowTitlesOn;
- (BOOL)setRowTitlesOn:(BOOL)on_off;
- (MiscTableTitleMode)rowTitleMode;
- (void)setRowTitleMode:(MiscTableTitleMode)x;
- (NXCoord)rowTitlesWidth;
- (void)setRowTitlesWidth:(NXCoord)x;

- (id)moveRowFrom:(int)fromPos to:(int)toPos;
- (int)rowPosition:(int)row;
- (int)rowAtPosition:(int)pos;

- (BOOL)sizeableRows;
- (BOOL)draggableRows;
- (BOOL)modifierDragRows;
- (NXCoord)uniformSizeRows;
- (NXCoord)rowAdjustedSize:(int)row;
- (NXCoord)rowSize:(int)row;
- (NXCoord)rowMinSize:(int)row;
- (NXCoord)rowMaxSize:(int)row;
- (NXCoord)rowDataSize:(int)row;
- (BOOL)rowIsSizeable:(int)row;
- (BOOL)rowExpandsToData:(int)row;
- (BOOL)rowIsAutosize:(int)row;
- (char const*)rowTitle:(int)row;
- (int)rowTag:(int)row;
- (MiscTableCellStyle)rowCellType:(int)row;
- (id)rowCellPrototype:(int)row;

- (void)setSizeableRows:(BOOL)flag;
- (void)setDraggableRows:(BOOL)flag;
- (void)setModifierDragRows:(BOOL)flag;
- (void)setUniformSizeRows:(NXCoord)uniform_size;

- (void)setRow:(int)row size:(NXCoord)size;
- (void)setRow:(int)row minSize:(NXCoord)size;
- (void)setRow:(int)row maxSize:(NXCoord)size;
- (void)setRow:(int)row dataSize:(NXCoord)size;
- (void)setRow:(int)row sizeable:(BOOL)flag;
- (void)setRow:(int)row expandsToData:(BOOL)flag;
- (void)setRow:(int)row autosize:(BOOL)flag;
- (void)setRow:(int)row title:(char const*)title;
- (void)setRow:(int)row tag:(int)tag;
- (void)setRow:(int)row cellType:(MiscTableCellStyle)type;
- (void)setRow:(int)row cellPrototype:(id)cell;

- (int)numVisibleRows;
- (int)firstVisibleRow;
- (int)lastVisibleRow;
- (BOOL)rowIsVisible:(int)n;
- (void)setFirstVisibleRow:(int)n;
- (void)setLastVisibleRow:(int)n;


// DRAWING --------------------------------------------------------------------
- (void)getDocClipFrame:(NXRect*)rect;

- (id)drawCellAt:(int)row :(int)col;					// Physical coords

- (id)drawRow:(int)row;									// Physical coords
- (id)drawCol:(int)col;									// Physical coords
- (id)border:(MiscBorderType)b drawSlot:(int)n;

- (int)numVisibleSlots:(MiscBorderType)b;
- (int)firstVisibleSlot:(MiscBorderType)b;
- (int)lastVisibleSlot:(MiscBorderType)b;
- (BOOL)border:(MiscBorderType)b slotIsVisible:(int)n;
- (void)border:(MiscBorderType)b setFirstVisibleSlot:(int)n;
- (void)border:(MiscBorderType)b setLastVisibleSlot:(int)n;

- (void)scrollCellToVisible:(int)row :(int)col;			// Physical coords
- (void)scrollRowToVisible:(int)row;					// Physical coord
- (void)scrollColToVisible:(int)col;					// Physical coord
- (id)scrollSelToVisible;

- (id)border:(MiscBorderType)b drawSlotTitle:(int)n;
- (id)drawRowTitle:(int)n;
- (id)drawColTitle:(int)n;

- (void)selectionChanged;		// Subclasses may want to override.
- (id)setAutodisplay:(BOOL)x;

- (id)sizeToCells;


// INTERNAL COMMUNICATIONS ON USER-ACTIONS FOR SUBCLASSES ONLY ----------------
// Private: BorderView -> TableScroll
- (void)border:(MiscBorderType)b slotDraggedFrom:(int)fromPos to:(int)toPos;
- (void)border:(MiscBorderType)b slotSortReversed:(int)n;
- (void)border:(MiscBorderType)b slotResized:(int)n;

@end

// SORTING --------------------------------------------------------------------
@interface MiscTableScroll(Sort)

- (MiscCompareSlotFunc)compareSlotFunc;
- (void)setCompareSlotFunc:(MiscCompareSlotFunc)f;

- (void)sortInfoInit:(MiscSlotSortInfo*)ip border:(MiscBorderType)b;
- (void)sortInfoDone:(MiscSlotSortInfo*)ip;

// Slots
- (BOOL)autoSortSlots:(MiscBorderType)b;
- (void)border:(MiscBorderType)b setAutoSortSlots:(BOOL)flag;
- (int const*)slotSortVector:(MiscBorderType)b len:(int*)len;
- (void)border:(MiscBorderType)b setSlotSortVector:(int const*)v len:(int)n;

- (void)sortSlots:(MiscBorderType)b;
- (BOOL)slotsAreSorted:(MiscBorderType)b;
- (BOOL)border:(MiscBorderType)b sortSlot:(int)n;
- (BOOL)border:(MiscBorderType)b slotIsSorted:(int)n;

- (int)border:(MiscBorderType)b compareSlots:(int)slot1 :(int)slot2
		info:(MiscSlotSortInfo*)ip;
- (int)border:(MiscBorderType)b compareSlots:(int)slot1 :(int)slot2;


- (MiscCompareEntryFunc)border:(MiscBorderType)b slotSortFunc:(int)n;
- (MiscSortDirection)border:(MiscBorderType)b slotSortDirection:(int)n;
- (MiscSortType)border:(MiscBorderType)b slotSortType:(int)n;
- (void)border:(MiscBorderType)b setSlot:(int)n
		sortFunc:(MiscCompareEntryFunc)x;
- (void)border:(MiscBorderType)b setSlot:(int)n
		sortDirection:(MiscSortDirection)x;
- (void)border:(MiscBorderType)b setSlot:(int)n
		sortType:(MiscSortType)x;

// Cols
- (BOOL)autoSortCols;
- (void)setAutoSortCols:(BOOL)flag;
- (int const*)colSortVectorLen:(int*)len;
- (void)setColSortVector:(int const*)v len:(int)n;

- (void)sortCols;
- (BOOL)colsAreSorted;
- (BOOL)sortCol:(int)n;
- (BOOL)colIsSorted:(int)n;
- (int)compareCols:(int)c1 :(int)c2 info:(MiscSlotSortInfo*)ip;
- (int)compareCols:(int)c1 :(int)c2;

- (MiscCompareEntryFunc)colSortFunc:(int)n;
- (MiscSortDirection)colSortDirection:(int)n;
- (MiscSortType)colSortType:(int)n;
- (void)setCol:(int)n sortFunc:(MiscCompareEntryFunc)x;
- (void)setCol:(int)n sortDirection:(MiscSortDirection)x;
- (void)setCol:(int)n sortType:(MiscSortType)x;

// Rows
- (BOOL)autoSortRows;
- (void)setAutoSortRows:(BOOL)flag;
- (int const*)rowSortVectorLen:(int*)len;
- (void)setRowSortVector:(int const*)v len:(int)n;

- (void)sortRows;
- (BOOL)rowsAreSorted;
- (BOOL)sortRow:(int)n;
- (BOOL)rowIsSorted:(int)n;
- (int)compareRows:(int)r1 :(int)r2 info:(MiscSlotSortInfo*)ip;
- (int)compareRows:(int)r1 :(int)r2;

- (MiscCompareEntryFunc)rowSortFunc:(int)n;
- (MiscSortDirection)rowSortDirection:(int)n;
- (MiscSortType)rowSortType:(int)n;
- (void)setRow:(int)n sortFunc:(MiscCompareEntryFunc)x;
- (void)setRow:(int)n sortDirection:(MiscSortDirection)x;
- (void)setRow:(int)n sortType:(MiscSortType)x;
@end


// DATA MANIPULATION ----------------------------------------------------------
@interface MiscTableScroll(Data)
- (BOOL)isLazy;
- (void)setLazy:(BOOL)flag;

- (id)cellAt:(int)row :(int)col;

// Lazy mode tables that perform multiple-buffering:
- (int)buffCount;

// Lazy mode asks delegate/dataDelegate first then uses cellAt::
// Eager mode goes straight to cellAt::
- (int)tagAt:(int)row :(int)col;
- (int)intValueAt:(int)row :(int)col;
- (float)floatValueAt:(int)row :(int)col;
- (double)doubleValueAt:(int)row :(int)col;
- (char const*)stringValueAt:(int)row :(int)col;
// For ButtonCells.
- (char const*)titleAt:(int)row :(int)col;
- (int)stateAt:(int)row :(int)col;

// Invoke delegate methods if possible.
- (id)reviveCell:(id)cell at:(int)row :(int)col;
- (id)retireCell:(id)cell at:(int)row :(int)col;

// Builtin default implementation.
- (id)doReviveCell:(id)cell at:(int)row:(int)col;
- (id)doRetireCell:(id)cell at:(int)row:(int)col;

- (id)addSlot:(MiscBorderType)b;
- (id)border:(MiscBorderType)b insertSlotAt:(int)pos;
- (id)border:(MiscBorderType)b deleteSlotAt:(int)pos;
- (int)numSlots:(MiscBorderType)b;

- (id)addCol;
- (id)insertColAt:(int)pos;
- (id)deleteColAt:(int)pos;
- (int)numCols;

- (id)addRow;
- (id)insertRowAt:(int)pos;
- (id)deleteRowAt:(int)pos;
- (int)numRows;

- (id)renewRows:(int)count;
- (id)empty;					// [self renewRows:0];
- (id)emptyAndFreeCells;
@end


// PASTEBOARD SUPPORT ---------------------------------------------------------
@interface MiscTableScroll(Pasteboard)
- (id)copy:(id)sender;
- (id)cut:(id)sender;
- (id)validRequestorForSendType:(NXAtom)t_write andReturnType:(NXAtom)t_read;
- (id)builtinValidRequestorForSendType:(NXAtom)t_write
		andReturnType:(NXAtom)t_read;
- (BOOL)canWritePboardType:(NXAtom)type;
- (BOOL)builtinCanWritePboardType:(NXAtom)type;
- (BOOL)writeSelectionToPasteboard:(id)pboard types:(NXAtom*)types;
- (BOOL)builtinWriteSelectionToPasteboard:(id)pboard types:(NXAtom*)types;
- (void)writePboard:(id)pboard type:(NXAtom)type toStream:(NXStream*)stream;
- (void)builtinWritePboard:(id)pb type:(NXAtom)type toStream:(NXStream*)stream;
- (void)writeNXAsciiPboardTypeToStream:(NXStream*)stream;
- (void)writeNXTabularTextPboardTypeToStream:(NXStream*)stream;
- (id)readSelectionFromPasteboard:(id)pboard;
- (id)builtinReadSelectionFromPasteboard:(id)pboard;
- (void)builtinRegisterServicesTypes;
- (void)registerServicesTypes;
@end


// INCREMENTAL SEARCH ---------------------------------------------------------
@interface MiscTableScroll(IncrementalSearch)
- (BOOL)incrementalSearch:(NXEvent*)p;
- (BOOL)doIncrementalSearch:(NXEvent*)p col:(int)col;
- (BOOL)getISearchCol:(int*)col;
- (BOOL)doGetISearchCol:(int*)col;
@end


// KEYBOARD EVENTS ------------------------------------------------------------
@interface MiscTableScroll(Keyboard)
- (id)keyDown:(NXEvent*)event;
@end


// EDITING --------------------------------------------------------------------
@interface MiscTableScroll(Edit)
- (id)getCellFrame:(NXRect*)rect at:(int)row :(int)col; // Physical coords
- (id)getRow:(int*)row andCol:(int*)col					// Physical coords
		forPoint:(NXPoint const*)point;
- (BOOL)getPreviousEditRow:(MiscCoord_P*)p_row andCol:(MiscCoord_P*)p_col;
- (BOOL)getNextEditRow:(MiscCoord_P*)p_row andCol:(MiscCoord_P*)p_col;
- (BOOL)getNext:(BOOL)foreward
		editRow:(MiscCoord_P*)p_row andCol:(MiscCoord_P*)p_col;
- (BOOL)isEditing;
- (BOOL)finishEditing;			// Normal, conditional termination.
- (void)abortEditing;			// Forcibly abort the editing session.
- (id)textDidEnd:(id)sender endChar:(unsigned short)whyEnd;
- (id)textDidChange:(id)sender;
- (id)textDidGetKeys:(id)sender isEmpty:(BOOL)flag;
- (BOOL)textWillChange:(id)sender;
- (BOOL)textWillEnd:(id)sender;
- (void)edit:(NXEvent const*)ev at:(MiscCoord_P)row :(MiscCoord_P)col;
- (BOOL)canEdit:(NXEvent const*)ev at:(MiscCoord_P)row :(MiscCoord_P)col;
- (BOOL)editIfAble:(NXEvent const*)ev at:(MiscCoord_P)row :(MiscCoord_P)col;
- (void)editCellAt:(MiscCoord_P)row :(MiscCoord_P)col;
@end


// PRINTING -------------------------------------------------------------------
@interface MiscTableScroll(Print)
- (id)printPSCode:(id)sender;
- (MiscTablePrintInfo const*)getPrintInfo;
- (id)getPageHeader;
- (id)getPageFooter;
- (void)setPageHeader:(id)obj;
- (void)setPageFooter:(id)obj;
@end


// DELEGATE PROTOCOL ----------------------------------------------------------
@interface MiscTableScroll(Delegate)
- (id)tableScroll:(MiscTableScroll*)scroll border:(MiscBorderType)b
		slotDraggedFrom:(int)fromPos to:(int)toPos;
- (id)tableScroll:(MiscTableScroll*)scroll border:(MiscBorderType)b
		slotSortReversed:(int)n;
- (id)tableScroll:(MiscTableScroll*)scroll border:(MiscBorderType)b
		slotResized:(int)n;

- (id)tableScroll:(MiscTableScroll*)scroll
		changeFont:(Font*)oldFont to:(Font*)newFont;
- (id)tableScroll:(MiscTableScroll*)scroll
		fontChangedFrom:(Font*)oldFont to:(Font*)newFont;

- (id)tableScroll:(MiscTableScroll*)scroll
		backgroundColorChangedTo:(NXColor)x;
- (id)tableScroll:(MiscTableScroll*)scroll
		highlightBackgroundColorChangedTo:(NXColor)x;
- (id)tableScroll:(MiscTableScroll*)scroll
		highlightTextColorChangedTo:(NXColor)x;
- (id)tableScroll:(MiscTableScroll*)scroll
		textColorChangedTo:(NXColor)x;

- (BOOL)tableScroll:(MiscTableScroll*)scroll getISearchCol:(int*)col;
- (int)tableScrollBuffCount:(MiscTableScroll*)scroll;

- (id)tableScroll:(MiscTableScroll*)scroll
		border:(MiscBorderType)b slotPrototype:(int)slot;
- (char const*)tableScroll:(MiscTableScroll*)scroll
		border:(MiscBorderType)b slotTitle:(int)slot;

- (id)tableScroll:(MiscTableScroll*)ts cellAt:(int)row :(int)col;
- (id)tableScroll:(MiscTableScroll*)ts reviveCell:(id)cell at:(int)r :(int)c;
- (id)tableScroll:(MiscTableScroll*)ts retireCell:(id)cell at:(int)r :(int)c;

- (int)tableScroll:(MiscTableScroll*)scroll tagAt:(int)row :(int)col;
- (int)tableScroll:(MiscTableScroll*)scroll intValueAt:(int)row :(int)col;
- (float)tableScroll:(MiscTableScroll*)scroll floatValueAt:(int)row :(int)col;
- (double)tableScroll:(MiscTableScroll*)scroll
		doubleValueAt:(int)row :(int)col;
- (char const*)tableScroll:(MiscTableScroll*)scroll
		stringValueAt:(int)row :(int)col;
- (char const*)tableScroll:(MiscTableScroll*)scroll titleAt:(int)row :(int)col;
- (int)tableScroll:(MiscTableScroll*)scroll stateAt:(int)row :(int)col;

- (BOOL)tableScroll:(MiscTableScroll*)scroll
		allowDragOperationAt:(int)row :(int)col;
- (void)tableScroll:(MiscTableScroll*)scroll
		preparePasteboard:(Pasteboard*)pb forDragOperationAt:(int)row:(int)col;
- (NXImage*)tableScroll:(MiscTableScroll*)scroll
		imageForDragOperationAt:(int)row :(int)col;
- (NXDragOperation)tableScroll:(MiscTableScroll*)scroll
		draggingSourceOperationMaskForLocal:(BOOL)isLocal;
- (BOOL)tableScrollIgnoreModifierKeysWhileDragging:(MiscTableScroll*)scroll;

- (void)tableScrollRegisterServicesTypes:(MiscTableScroll*)scroll;
- (id)tableScroll:(MiscTableScroll*)scroll
		validRequestorForSendType:(NXAtom)t_write
		andReturnType:(NXAtom)t_read;
- (BOOL)tableScroll:(MiscTableScroll*)scroll
		canWritePboardType:(NXAtom)type;
- (void)tableScroll:(MiscTableScroll*)scroll
		writePboard:(id)pboard type:(NXAtom)type toStream:(NXStream*)stream;
- (BOOL)tableScroll:(MiscTableScroll*)scroll
		writeSelectionToPasteboard:(id)pboard types:(NXAtom*)types;
- (id)tableScroll:(MiscTableScroll*)scroll
		readSelectionFromPasteboard:(id)pboard;

- (void)tableScrollWillPrint:(MiscTableScroll*)scroll;
- (void)tableScroll:(MiscTableScroll*)scroll didPrint:(BOOL)flag;
- (void)tableScroll:(MiscTableScroll*)scroll
		willPrintPageHeader:(id)pageHeader
		info:(MiscTablePrintInfo const*)info;
- (void)tableScroll:(MiscTableScroll*)scroll
		willPrintPageFooter:(id)pageFooter
		info:(MiscTablePrintInfo const*)info;

- (BOOL)tableScroll:(MiscTableScroll*)scroll canEdit:(NXEvent const*)ev
		at:(int)row :(int)col;
- (void)tableScroll:(MiscTableScroll*)scroll edit:(NXEvent const*)ev
		at:(int)row :(int)col;
- (BOOL)tableScroll:(MiscTableScroll*)scroll setStringValue:(char const*)s
		at:(int)row :(int)col;
- (void)tableScroll:(MiscTableScroll*)scroll abortEditAt:(int)row :(int)col;
- (void)tableScroll:(MiscTableScroll*)scroll willEditAt:(int)row :(int)col;
- (void)tableScroll:(MiscTableScroll*)scroll didEdit:(BOOL)changed 
		at:(int)row :(int)col;
@end


// DATA CELL PROTOCOL ---------------------------------------------------------
@interface MiscTableScroll(DataCell)
- (id)tableScroll:(MiscTableScroll*)scroll reviveAt:(int)row :(int)col;
- (id)tableScroll:(MiscTableScroll*)scroll retireAt:(int)row :(int)col;
@end

#endif // __MiscTableScroll_h
