//
//	MiscPopUpButton.m -- convenience items to Pop Up Lists
//		Written by Stefan Boehringer Copyright (c) 1996 by Stefan Boehringer.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	
// This is useful for handling popups programmatically
// Sat Dec 16 11:02:52 MET 1995

#import <misckit/misckit.h>

@implementation Button(PopUpButton)

// replace with indexOfItem
- (int)positionOfTitle:(const char *)seaTitle
{	Matrix		*matrix=[[self target] itemList];
	int			i;

	for ([matrix getNumRows:&i numCols:0]; --i>=0; )
		if (!strcmp([[matrix cellAt:i :0] title],seaTitle))
			break;
	return i;
}
- (int)positionOfTag:(int)thetag
{	Matrix		*matrix=[[self target] itemList];
	int			i;

	for ([matrix getNumRows:&i numCols:0]; --i>=0; )
		if([[matrix cellAt:i :0] tag] == thetag)
			break;
	return i;
}

// workaround for NeXT-bug
- (int)popUpTag
{	Matrix		*matrix=[[self target] itemList];

	if(![matrix selectedCell])
		[matrix selectCellAt:[self positionOfTitle:[self title]] :0];

	return [[matrix selectedCell] tag];
}

- (void)selectCellAt:(int)position
{	Matrix		*matrix=[[self target] itemList];
	[matrix selectCellAt:position :0];
	[self setTitle:[[matrix selectedCell] title]];
}

- selectTitle:(const char *)seaTitle
{	if(!seaTitle) return nil;
	[self selectCellAt:[self positionOfTitle:seaTitle]];
	return self;
}
- selectTag:(int)thetag
{	[self selectCellAt:[self positionOfTag:thetag]];
	return self;
}
-(int) selectedTag
{	return [self popUpTag];
}

- buildItemsFromString:(const char*) str separator:(char) sep underscoreReplace:(BOOL) repl
			  addEmpty:(BOOL) add
{	id			 popUp=[self target];
	char		 sepStr[2]={sep,0};
	MiscString	*tmpString=[[MiscString alloc] initString:str];
	List		*lst=[tmpString tokenize:sepStr into:nil];
	int			 i;
	for (i=[popUp count]; --i>=0; ) [popUp removeItemAt:0];
	for (i=[lst count]; --i>=0; )
	{	if(repl) [[lst objectAt:i] replaceEveryOccurrenceOfChar:'_' withChar:' '];
		[popUp insertItem:[[lst objectAt:i] stringValue] at:0];
	} if(add) [popUp insertItem:" " at:0];
	[self selectCellAt:0];
	[[lst freeObjects] free];[tmpString free];
	[self selectCellAt:0];
	return self;
}

- buildItemsFromString:(const char*) str separator:(char) sep
{	return [self buildItemsFromString:str separator:sep underscoreReplace:NO addEmpty:NO];
}

@end

