//
//	MiscTextExtensions.m -- extensions to the NeXT Text object
//		Originally written by Scott Anguish
//		Copyright (c) 1994 by Scott Anguish.
//		Modified and extended by Don Yacktman for inclusion into the MiscKit.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/misckit.h>

@implementation Text(MiscExtensions)

- (MiscString *)getContentString
{
	NXStream *theStream;
	char *data;
	int length, maxLength;
	MiscString *theString;

	theString = [[MiscString alloc] init];
	theStream = NXOpenMemory(NULL, 0, NX_READWRITE);
	[self writeText:theStream];
	NXPutc(theStream,'\0');
	NXGetMemoryBuffer(theStream, &data, &length, &maxLength);
	[theString setStringValue:data];
	NXClose(theStream);
	return theString;
}
	
	
- (MiscString *)getSelectionString
{
	NXSelPt start, end;

	[self getSel:&start :&end];
	return [self getSubstringStringStart:start.cp length:(end.cp - start.cp)];
}
	
	
- (MiscString *)getSubstringStringStart:(int)startPos length:(int)numChars
{
	int i;
	char *temp = malloc(numChars + 1);
	MiscString *stringObj;

	i = [self getSubstring:temp start:startPos length:numChars];
	temp[i] = '\0'; // make sure it is terminated
	stringObj = [MiscString newWithString:temp];
	free(temp);
	return stringObj;
}

@end



// toDo		ctrl-Backspace: delete line
//			shiftCrsrUp/dwn: pagewise scrolling


@interface Object(MiscTexDelegateExtensions)
-(BOOL) escapePressed:sender;
@end

int getCharOfText(Text *self,int pos)
{	char text[2];
	if([self getSubstring:text start:pos length:1] != 1) return -1;
	return text[0];
}

@implementation Text(MiscDExtensions)
// donated by Stefan Bhringer

-(char*) textAsCopy
{	int	  theLen=[self textLength];
	char *buf=malloc(theLen+1);
	[self getSubstring:buf start:0 length:theLen+1];
	return buf;
}
-(MiscString*) textAsString
{	char *buf=[self textAsCopy];
	MiscString *theStr=[[MiscString alloc] initString:buf];
	free(buf);
	return theStr;
}
-(char*) selectedTextAsCopy
{	int	  theLen=spN.cp-sp0.cp;
	char *buf=malloc(theLen+1);
	[self getSubstring:buf start:sp0.cp length:theLen];
	buf[theLen]=0;
	return buf;
}
- selectWordUnderCaret
{	int lpos=sp0.cp,rpos=spN.cp,c;
	if(lpos ==rpos)
	{	while((c=getCharOfText(self,rpos))>= 0 && !NXIsSpace(c)) rpos++;
		if(NXIsSpace( getCharOfText(self,lpos))) lpos--;
		while(lpos && (c=getCharOfText(self,lpos))>= 0 && !NXIsSpace(c)) lpos--;
		if(NXIsSpace(c)) lpos++;
		[self setSel:lpos:rpos];
	}
	return self;
}


-(int) lengthOfLineForSel:(NXSelPt *)sel
{	return theBreaks->breaks[sel->line/sizeof(NXLineDesc)]&0x3fff;
}
@end

//	keyFilter stuff.
// Extensions to the Textobject that give convenient access to the text
// and a writenow like keyfilter (which is far from perfect by the way)

NXCharFilterFunc oldFilter;

void Misc_installCharFilter(Text *self,NXCharFilterFunc newFilter)
{	oldFilter=[self charFilter];
	[self setCharFilter:newFilter];
}

unsigned short Misc_WriteNowCharfilter(unsigned short charCode,int keyFlags,unsigned short charSet)
{	int pos,c;
	Text	*currText=[[NXApp keyWindow] firstResponder];
	NXSelPt sp0,spN;
	[currText getSel:&sp0:&spN];
	switch(NXFieldFilter(charCode,0,charSet))	// dont set it see the flags (esp.cntrl)
	{	case NX_LEFT:
			if(keyFlags & NX_SHIFTMASK)
			{	if(sp0.cp == sp0.c1st) return 0;
				[currText setSel:sp0.c1st+1:sp0.c1st+1];
			} else if(keyFlags & NX_CONTROLMASK)
			{	pos=sp0.cp;
				while(pos && (c=getCharOfText(currText,--pos))>= 0 && !NXIsSpace(c)){}
				[currText setSel:pos+1:pos+1];
				return NX_LEFT;		// default cntrl-crsr lets arrowos be inserted into text
			}
		break;
		case NX_RIGHT:
			if(keyFlags & NX_SHIFTMASK)
			{	pos=spN.c1st+[currText lengthOfLineForSel:&spN]-1;
				if(pos == spN.cp) return 0;
				[currText setSel:pos-1:pos-1];
			} else if(keyFlags & NX_CONTROLMASK)
			{	pos=spN.cp;
				while((c=getCharOfText(currText,++pos))>= 0 && !NXIsSpace(c)){}
				[currText setSel:pos-1:pos-1];
				return NX_RIGHT;
			}
		break;
		case NX_UP:
			if(keyFlags & NX_SHIFTMASK)		//pageUp
			{
			}
		break;
		case NX_DOWN:
			if(keyFlags & NX_SHIFTMASK)		//pageUp
			{
			}
		break;
		case NX_DELETE:
			if(keyFlags & NX_CONTROLMASK)	// delete curr. line
			{
			}
		break;
		case 32:	//esc or spc
		{	id delegate=[currText delegate];
			if(charCode == 32 && !(keyFlags & NX_COMMANDMASK)) break;
			if([delegate respondsTo:@selector(escapePressed:)])
			{	if(![delegate escapePressed:currText])
					return 0;
				else keyFlags&=~NX_COMMANDMASK;
			}
		} break;
	} return oldFilter(charCode,keyFlags,charSet);
}



// Written by Erik M. Buck and posted to USENET:

@implementation Text(MiscPrintf)

- appendText:(const char *)aString
{
    int e;

    if ((e=[self textLength])<=0) [self setText:aString];
    else [[self setSel:e:e] replaceSel:aString];
    [self scrollSelToVisible];
    return self;
}

- printf:(const char *)format,...
{
    NXStream *s;
    char *streambuf;
    int len, maxlen;
    va_list ap;

    va_start(ap, format);
    s=NXOpenMemory(NULL, 0, NX_WRITEONLY);
    NXVPrintf(s, format, ap);
    (void)NXPutc(s, '\0');
    NXFlush(s);
    NXGetMemoryBuffer(s, &streambuf, &len, &maxlen);
    [self appendText:streambuf];
    NXCloseMemory(s, NX_FREEBUFFER);
    va_end(ap);
    return 0;
}

- (char *)stringValue
{
    char *buf;
    buf = NXZoneMalloc([NXApp zone], [self textLength]+1);
    [self getSubstring:buf start:0 length:[self textLength]+1];
    return buf;
}

- (void)setStringValue:(char *)string
{
    [self setText:string];
    return;
}


@end

