//
//	_MiscGenericLoopPlaceHolder.m -- abstract container used by loop commands
//		Written by Don Yacktman Copyright (c) 1996 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import "_MiscGenericLoopPlaceHolder.h"

@implementation _MiscGenericLoopPlaceHolder

- setVariableName:(MiscString *)aName
{
	if (variableName) [variableName free];
	variableName = aName;
	return self;
}

- setTagName:(MiscString *)aName
{
	if (tag) [tag free];
	tag = aName;
	return self;
}

- setJumpback:(_MergeEngineTag)aPlace { jumpback = aPlace; return self; }

- (MiscString *)variableName { return variableName; }
- (MiscString *)tagName { return tag; }
- (_MergeEngineTag)jumpback { return jumpback; }

- setPosition:(int)aPosition { position = aPosition; return self; }
- (int)position { return position; }

- free
{
	[tag free];
	[variableName free];
	return [super free];
}

@end


@implementation MiscMergeEngine (_MiscMergeLoopCommand)

// This is a "special" variable available to all mergers; we make
// sure that it exists.  Commands can obviously still install their
// own variables as well...
- _loopStack
/*" Returns the special loop stack merge variable, creating it if
necessary.  The loop stack is used by the loop/endloop commands.
"*/
{
	static id loopStackKey = nil;
	id loopStack;

	if (!loopStackKey)
		loopStackKey = [MiscString newWithString:"_Misc_loopStack"];
	loopStack = [self getVariableNamed:loopStackKey];
	if (!loopStack) {
		loopStack = [[MiscStack alloc] init];
		[self setVariableNamed:loopStackKey to:loopStack];
	}
	return loopStack;
}

@end
