//
//	_MiscMergeEndforeachCommand.m -- 
//		Written by Don Yacktman Copyright (c) 1996 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import <misckit/miscmerge.h>
#import "_MiscMergeEndforeachCommand.h"
#import "_MiscForEachPlaceHolder.h"
#import "_MiscLoopPlaceHolder.h"

@implementation _MiscMergeEndforeachCommand

static MiscString *myKey = nil;

- parseFromString:(MiscString *)aString
{
	if (!myKey) {
		myKey = [MiscString new];
		[myKey setStringValue:"endforeach"];
	}
	[self eatKeyWord:myKey from:aString isOptional:NO];
	tagString = [self getArgumentStringFrom:aString toEnd:YES];
	return self;
}

- executeForMerge:(MiscMergeEngine *)aMerger
{
	BOOL loopDone = NO;
	id topLoop = [[aMerger _loopStack] topObject];

	if (![topLoop isKindOf:[_MiscForEachPlaceHolder class]]) {
		// if wrong class of place holder is on the stack...
		fprintf(stderr, "%s:  endforeach must match with a foreach command.\n", [[self class] name]);
		loopDone = YES;
	} else {	
		// Does current loop placeholder match our tag?
		if ([tagString compare:[topLoop tagName]]) {
			// No, so we have an error.  Bail out.
			fprintf(stderr, "%s:  tags %s and %s do not match.\n", [[self class] name], [tagString stringValue], [[topLoop tagName] stringValue]);
			loopDone = YES;
		} else { // Yes.  Continue.
			int nextPos = [topLoop position] + 1;
			id nextValue = [[topLoop list] objectAt:nextPos];

			[topLoop setPosition:nextPos];
			[aMerger setSymbol:[topLoop variableName] toValue:[nextValue copy]];

			// See if the loop is done yet
			if ((nextPos >= [[topLoop list] count]) || (!nextValue)) loopDone = YES;
		}
	}
	if (loopDone) { // Are we out of the loop yet?
		// Yes.  Pop top guy from stack.
		[[[aMerger _loopStack] popObject] free];
	} else { // No.
		// Make the engine jump to the start of the loop for another pass
		[aMerger goToCommand:[topLoop jumpback]];
	}
	return self;
}

@end



