//
//	_MiscMergeLoopCommand.m -- 
//		Written by Don Yacktman Copyright (c) 1996 by Don Yacktman.
//				Version 1.0.  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	


#import <misckit/miscmerge.h>
#import "_MiscMergeLoopCommand.h"
#import "_MiscLoopPlaceHolder.h"

@implementation _MiscMergeLoopCommand

static MiscString *myKey = nil;

- parseFromString:(MiscString *)aString
{
	if (!myKey) {
		myKey = [MiscString new];
		[myKey setStringValue:"loop"];
	}
	[self eatKeyWord:myKey from:aString isOptional:NO];
	if (varString) [varString free];
	if (startString) [startString free];
	if (endString) [endString free];
	if (stepString) [stepString free];
	if (tagString) [tagString free];
	varString = [self getArgumentStringFrom:aString toEnd:NO];
	startString = [self getArgumentStringFrom:aString toEnd:NO];
	endString = [self getArgumentStringFrom:aString toEnd:NO];
	stepString = [self getArgumentStringFrom:aString toEnd:NO];
	tagString = [self getArgumentStringFrom:aString toEnd:YES];

	// look for possible trouble:
	{
		int step = [stepString intValue];
		int start = [startString intValue];
		int end = [endString intValue];
	
		if (step == 0) {
			fprintf(stderr, "%s:  Loop %s is infinite.  Setting to 1 or -1 as appropriate.\n", [[self class] name], [tagString stringValue]);
			if (start <= end) [stepString setStringValue:"1"];
			else [stepString setStringValue:"-1"];
		}
		if (((start < end) && (step < 0)) || ((start > end) && (step > 0))) {
			fprintf(stderr, "%s:  Loop %s is probably longer than you want.  (start,end,step) = (%d,%d,%d).\n", [[self class] name], [tagString stringValue], start, end, step);
		}
	}
	return self;
}

- executeForMerge:(MiscMergeEngine *)aMerger
{	// all we have to do here is set up the loop.  The end command is
	// where all the real work is done.
	id placeHolder = [[_MiscLoopPlaceHolder alloc] init];

	// Build the place holder
	[placeHolder setStart:[startString intValue]];
	[placeHolder setPosition:[placeHolder start]];
	[placeHolder setEnd:[endString intValue]];
	[placeHolder setStep:[stepString intValue]];
	[placeHolder setVariableName:[varString copy]];
	[placeHolder setTagName:[tagString copy]];
	[placeHolder setJumpback:[aMerger currentCommand]];

	// Push it onto the stack
	[[aMerger _loopStack] pushObject:placeHolder];

	// Set the merge variable
	[aMerger setSymbol:[placeHolder variableName] toValue:startString];

	return self;
}

@end
