//
//	MiscStringComparing.m
//		Written by Don Yacktman Copyright (c) 1993 by Don Yacktman.
//				Version 1.95  All rights reserved.
//		This notice may not be removed from this source code.
//
//	This object is included in the MiscKit by permission from the author
//	and its use is governed by the MiscKit license, found in the file
//	"LICENSE.rtf" in the MiscKit distribution.  Please refer to that file
//	for a list of all applicable permissions and restrictions.
//	

#import <misckit/MiscString.h>

@implementation MiscString(Comparing)

// This category is composed of methods which compare
// MiscStrings with other strings.

- (BOOL)isEqual:anObject
{
	if (anObject == self) return YES;
	// doesn't have to be a MiscString object to be equal...
	if ([anObject respondsTo:@selector(stringValue)]) {
		if (!NXOrderStrings((unsigned char *)buffer,
				(unsigned char *)[anObject stringValue],
				YES, -1, orderTable)) return YES;
	}
	return NO;
}

- (unsigned int)hash
// Use the NextStep string hashing functiuon NXStrHash()  instead of 
// the pointer hashing function. 
{
	return NXStrHash(NULL, buffer);
}

- (int)compareTo:sender
{
	return [self compareTo:sender n:(-1) caseSensitive:YES];
}

- (int)compareTo:sender n:(int)n
{
	return [self compareTo:sender n:n caseSensitive:YES];
}

- (int)cmp:(const char *)aString
{
	if (!aString && !buffer) return 0;		// both NULL, so "equal"
	if (!aString) return 1;	// only one is NULL, so not equal
	if (!buffer) return -1;	// only one is NULL, so not equal
	return strcmp(buffer, aString);
}

- (int)cmp:(const char *)aString n:(int)n
{
	if (!aString && !buffer) return 0;		// both NULL, so "equal"
	if (!aString) return 1;	// only one is NULL, so not equal
	if (!buffer) return -1;	// only one is NULL, so not equal
	// we don't check n; use strncmp's behavior here.
	return strncmp(buffer, aString, n);
}

- (int)compareTo:sender caseSensitive:(BOOL)sense
{
	return [self compareTo:sender n:(-1) caseSensitive:sense];
}

- (int)compareTo:sender n:(int)n caseSensitive:(BOOL)sense
{
	const char *s; int ret, senderLength, size = n;
	if (!sender) {
		if (!buffer) return 0;
		return 1;
	}
	if (![sender respondsTo:@selector(stringValue)]) return 1; // !=
	s = [sender stringValue];
	if (!buffer && !s) return 0;		// both are NULL
	if (!buffer) return -1;	// buffer is NULL
	if (!s) return 1;	// sender is NULL
	if ([sender respondsTo:@selector(length)]) senderLength = [sender length];
	else senderLength = strlen(s);
	if ((size > length) || (size > senderLength))  {
		size = MIN(length, senderLength);
		if (!size) if (senderLength) return -1;
		else if (length) return 1;
		else return 0;
	}
	ret = NXOrderStrings((unsigned char *)buffer,
			(unsigned char *)s, sense, size, orderTable);
	if (((size > length) || (size > senderLength)) && (!ret)) {
		if (length < senderLength) return -1;
		if (length > senderLength) return 1;
	}
	return ret;
}

- (int)casecmp:(const char *)aString
{
	if (!aString && !buffer) return 0;		// both NULL, so "equal"
	if (!aString || !buffer) return -1;	// only one is NULL, so not equal
	return strcasecmp(buffer, aString);
}

- (int)casecmp:(const char *)aString n:(int)n
{
	if (!aString && !buffer) return 0;		// both NULL, so "equal"
	if (!aString || !buffer) return -1;	// only one is NULL, so not equal
	// we don't check n; use strncasecmp's behavior here.
	return strncasecmp(buffer, aString, n);
}

- (int)endcmp:(const char *)aString
{
  return [self endcmp:aString n:-1];
}

- (int)endcmp:(const char *)aString n:(int)n
{
  id tempStr;
  int retval;
  if (!aString) return -1;
  tempStr = [[self class] newWithString:aString];
  retval = [self endCompareTo:tempStr n:n caseSensitive:YES];
  [tempStr free];
  return retval;
}

- (int)endcasecmp:(const char *)aString
{
  return [self endcasecmp:aString n:-1];
}

- (int)endcasecmp:(const char *)aString n:(int)n
{
  id tempStr;
  int retval;
  if (!aString) return -1;
  tempStr = [[self class] newWithString:aString];
  retval = [self endCompareTo:tempStr n:n caseSensitive:NO];
  [tempStr free];
  return retval;
}

- (int)endCompareTo:(id)sender
{
  return [self endCompareTo:sender n:-1 caseSensitive:YES];
}

- (int)endCompareTo:(id)sender caseSensitive:(BOOL)sense
{
  return [self endCompareTo:sender n:-1 caseSensitive:sense];
}

- (int)endCompareTo:(id)sender n:(int)n
{
  return [self endCompareTo:sender n:n caseSensitive:YES];
}

- (int)endCompareTo:(id)sender n:(int)n caseSensitive:(BOOL)sense
{ // DAY: fixed it: sender doesn't have to be a MiscString anymore.
	id tempStr, temp2, temp3;
	int retval, smallLen, senderLength;
	const char *s;
	// unlike MOKit, we allow NULL/empty strings, and we adjust n if
	// it is too big.  also, this method is slower than MOKit complement.
	if (!sender) {
		if (!buffer) return 0;
		return 1;
	}
	if (!([sender respondsTo:@selector(stringValue)])) return -1;
	s = [sender stringValue];
	if (!s || !buffer) return -1;
	if ([sender respondsTo:@selector(length)]) senderLength = [sender length];
	else senderLength = strlen(s);
	// find the shorter of the two lengths
	smallLen = ((length >= senderLength) ? senderLength : length);
	if ((n == -1) || (n > smallLen)) n = smallLen;
	tempStr = [self right:n];
	if ([sender respondsTo:@selector(right:)]) temp2 = [sender right:n];
	else {
		temp3 = [[self class] newWithString:[sender stringValue]];
		temp2 = [temp3 right:n];
		[temp3 free];
	}
	retval = [tempStr compareTo:temp2 caseSensitive:sense];
	[tempStr free];
	[temp2 free];
	return retval;
}

//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--
// Levenstein string distancing:
//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--

#define	MAX_L	12
#define	RPL	1
#define	DEL	1
#define	INS	1
#define	SWP	1

#define	Mat(i)	(mat[i]&0x7fff)
#define	pSwp	0x8000
#define	msk(a)	(a&=0x7fff)
#define	isSwp(v)	((v)&pSwp)
#define	dVal(v)	((v)&0x7fff)

static int	min(int a, int b, int c)
{	msk(a),msk(b),msk(c);
	return a<b? (a<c? a: (b<c? b: c)): (b<c? b: (c<a? c: a));
}

/*
static int	minW(int a, int b, int c,short *which)
{	msk(a),msk(b),msk(c);
	return a<b? (a<c? *which=0,a: (b<c? *which=1,b: (*which=2,c))):
		(b<c? *which=1,b: (c<a? *which=2,c: (*which=0,a)));
}
*/

//cost2transf s1 into s2
#define	pIns	dist[-1]
/*
static int	distanceO2(const char *s1,const char *s2,short limit,short rpl,short ins,short del)
{	short	i,i1,i2,l1=strlen(s1),l2=strlen(s2),mat[l2+1],pRpl,pDel,*dist,minC;
	for (i=0; i<=l2; i++) mat[i]=i*ins;
	for (i1=0; i1<l1; i1++)
	{	for (i2=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+del; i2<l2; i2++,dist++,pRpl=pDel)
		{	pDel=*dist,*dist=min(pRpl+(s2[i2]==s1[i1]? 0: rpl),pDel+del,pIns+ins);
			if (*dist<minC) minC=*dist;
		} if (minC>limit) return minC;		//<*>minC is lower limit for dist.
	} return dist[-1];
}
*/

//	<N>will be correct and sensible only for swp<=2*rpl
static int	distanceO2S(const char *s1,const char *s2,short limit,short rpl,short ins,
	short del,short swp)
{	short	i,i1,i2,l1=strlen(s1),l2=strlen(s2),mat[l2+1],pRpl,pDel,*dist=0,minC,dSwp;
	for (i=0; i<=l2; i++) mat[i]=i*ins;

	for (i1=0; i1<l1; i1++)
	{	for (i2=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+del; i2<l2; i2++,dist++,pRpl=pDel)
		{	dSwp=0,pDel=*dist,*dist=min(pRpl+((isSwp(pRpl) || s2[i2]==s1[i1])? 0:
				(s2[i2+1]==s1[i1] && s2[i2]==s1[i1+1]? (dSwp=1,swp): rpl)),
					pDel+del,pIns+ins);
			if (dSwp) *dist|=pSwp;
			if (*dist<minC) minC=*dist;
		}
		if (minC>limit) return minC;		//<*>minC is lower limit for dist.
	}
	return dist[-1];
}

//	is s1 in s2
typedef struct {
	short	rpl,ins,del;
	int		testedLn,minLn;
} DistKtxt;
#define	Rpl	k->rpl
#define	Ins	k->ins
#define	Del	k->del
static int	distanceIn(const char *s1,const char *s2,int l1,int l2,short limit,DistKtxt *k)
{	int		i,i1,i2,mat[l1+1],pRpl,pDel,*dist,minC,mRet=limit+1,mLn=0;
	for (i=0; i<=l1; i++) mat[i]=i*Del;
	for (i2=0; i2<l2; i2++)
	{	for (i1=0,dist=mat+1,pRpl=mat[0],minC=mat[0]=pRpl+Ins; i1<l1; i1++,dist++,pRpl=pDel)
		{	pDel=*dist,*dist=min(pRpl+(s2[i2]==s1[i1]? 0: Rpl),pDel+Ins,pIns+Del);
			if (*dist<minC) minC=*dist;
		}
		if (minC>limit) return k->testedLn=i2+1,minC;//<*>! minC isLow Lim4dist.
		if (dist[-1]<=mRet) mRet=dist[-1],mLn=i2+1;
		if (dist[-1]<=minC) break;	//nothing to expect now
	} return k->testedLn=mLn,mRet;
}

static int	overlap(const char *s1,const char *s2,int minOvlp,int limit,DistKtxt *dk,
	int *pos1,int *pos2)
{	int		l1=strlen(s1),l2=strlen(s2),p1,p2,i,mx,dist=limit+1,fndDist=limit+1,nb;

	for (*pos1=*pos2=-1,i=-l1+minOvlp,mx=l2-minOvlp; i<=mx; i++)
	{	p1=i<0? -i: 0, p2=i<0? 0: i;
		if ((dist=distanceIn(s1+p1,s2+p2,l1-p1,l2-p2,limit,dk))<=limit)
		{	if (nb=dist<fndDist) fndDist=dist,*pos1=p1,*pos2=p2,dk->minLn=dk->testedLn;
			if (!dist || !nb) return fndDist;
		}
	}
	return dist;
}

-(unsigned) levensteinDistanceFor:(const char *) str cancelBeyond:(int) maxModifications
				   matchSubstring:(BOOL) ssFlag caseSensitive:(BOOL) caseFlag
{	int dummy;
	DistKtxt dk={RPL,INS,DEL};
	if(!caseFlag)
	{	int erg;
		MiscString *bufferStr=[self copy],*strStr=[[MiscString alloc] initString:str];
		[bufferStr toUpper];[strStr toUpper];
		erg=(  ssFlag ? overlap([strStr stringValue],[bufferStr stringValue],[strStr length],maxModifications,&dk,&dummy,&dummy):
					distanceO2S([strStr stringValue],[bufferStr stringValue],maxModifications, RPL,INS,DEL,SWP));
		[bufferStr free];[strStr free];
		return erg;
	} else
	{	return (ssFlag ? overlap(str,buffer,strlen(str),maxModifications,&dk,&dummy,&dummy):
					 distanceO2S(str,buffer,maxModifications, RPL,INS,DEL,SWP));
	}
}

//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--
// Additions for the MiscCompare, MiscEndCompare protocols:
//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--

// We should eventually decide upon a single protocol rather than having
// both the above and these methods.  This is an API bug in a big way.

- (int)compare:anObject length:(unsigned long)len ignoreCase:(BOOL)flag
{
	return [self compareTo:anObject n:len
			caseSensitive:(flag ? NO : YES)];
}

- (int)compareLiteral:(const char *)literal length:(unsigned long)len ignoreCase:(BOOL)flag
{
	MiscString *aString = [[self class] newWithString:literal];
	int ret = [self compareTo:aString n:len
			caseSensitive:(flag ? NO : YES)];
	[aString free];
	return ret;
}

- (int)endCompare:anObject length:(unsigned long)len ignoreCase:(BOOL)flag
{
	return [self endCompareTo:anObject n:len
			caseSensitive:(flag ? NO : YES)];
}

- (int)endCompareLiteral:(const char *)literal length:(unsigned long)len ignoreCase:(BOOL)flag
{
	MiscString *aString = [[self class] newWithString:literal];
	int ret = [self endCompareTo:aString n:len
			caseSensitive:(flag ? NO : YES)];
	[aString free];
	return ret;
}

//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--
// convenience methods of protocol that call the above four methods:
//--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--=--

- (int)compare:anObject
{
	return [self compare:anObject length:[self length] ignoreCase:NO];
}

- (int)compare:anObject ignoreCase:(BOOL)flag
{
	return [self compare:anObject length:[self length] ignoreCase:flag];
}

- (int)compare:anObject length:(unsigned long)len
{
	return [self compare:anObject length:len ignoreCase:NO];
}

- (int)compareLiteral:(const char *)literal
{
	return [self compareLiteral:literal length:[self length] ignoreCase:NO];
}

- (int)compareLiteral:(const char *)literal ignoreCase:(BOOL)flag
{
	return [self compareLiteral:literal length:[self length] ignoreCase:flag];
}

- (int)compareLiteral:(const char *)literal length:(unsigned long)len
{
	return [self compareLiteral:literal length:len ignoreCase:NO];
}

- (int)endCompare:anObject
{
	return [self endCompare:anObject length:[self length] ignoreCase:NO];
}

- (int)endCompare:anObject ignoreCase:(BOOL)flag
{
	return [self endCompare:anObject length:[self length] ignoreCase:flag];
}

- (int)endCompare:anObject length:(unsigned long)len
{
	return [self endCompare:anObject length:len ignoreCase:NO];
}

- (int)endCompareLiteral:(const char *)literal
{
	return [self endCompareLiteral:literal length:[self length] ignoreCase:NO];
}

- (int)endCompareLiteral:(const char *)literal ignoreCase:(BOOL)flag
{
	return [self endCompareLiteral:literal length:[self length]
			ignoreCase:flag];
}

- (int)endCompareLiteral:(const char *)literal length:(unsigned long)len
{
	return [self endCompareLiteral:literal length:len ignoreCase:NO];
}

@end
