// -*- objc -*-

#import <appkit/View.h>
#import <appkit/NXImage.h>

@interface MiscImageDissolver : View /*" Responder : Object "*/
{
    NXImage *sourceImage;   /*" A pointer to the image currently displayed. "*/
    NXImage *destImage;     /*" A pointer to the image currently dissolved to. "*/
    List *imageList;        /*" A pointer to the image list currently dissolved over. "*/

    int numSteps;        /*" The number of steps of the dissolving process. "*/
    double secs;         /*" The number of seconds between two steps. "*/
    
    int currentStep;     /*" The current dissolving step or < 0 if finished. "*/
    int currentIndex;    /*" The current index in imageList of the
			   image currently dissolved to. "*/

    char *image1Name;    /*" The name of an image "*/
    char *image2Name;    /*" The name of another image "*/
    
    DPSTimedEntry timedEntry;  /*" The timed entry for the dissolving process. "*/
}

/*" Initializing and freeing "*/
- initFrame: (const NXRect*) rect;
- free;

/*" Dissolving parameters "*/
- (int) numSteps;
- setNumSteps: (int) theNumSteps;

- (double) stepInterval;
- setStepInterval: (double) theSeconds;

/*" Convenience "*/
- (const char*) image1Name;
- setImage1Name: (const char*) theName;

- (const char*) image2Name;
- setImage2Name: (const char*) theName;

/*" The image "*/
- (NXImage*) image;
- setImage: (NXImage*) theImage;

/*" Dissolving "*/
- dissolveToImage: (NXImage*) theImage;
- dissolveOverImageList: (List*) theList;

- dissolveToImage1: sender;
- dissolveToImage2: sender;
- dissolveToOtherImage: sender;
- dissolveBackAndForth: sender;

- (BOOL) hasFinished;

/*" Displaying "*/
- drawSelf: (const NXRect*) rects : (int) rectCount;

- dissolveStep;

/*" Archiving "*/
- read: (NXTypedStream*) theStream;
- write: (NXTypedStream*) theStream;

@end
