#ifndef __MiscTableView_h
#define __MiscTableView_h
//=============================================================================
//
//		Copyright (C) 1995,1996 by Paul S. McCarthy and Eric Sunshine.
//				Written by Paul S. McCarthy and Eric Sunshine.
//							All Rights Reserved.
//
//		This notice may not be removed from this source code.
//
//		This object is included in the MiscKit by permission from the authors
//		and its use is governed by the MiscKit license, found in the file
//		"License.rtf" in the MiscKit distribution.	Please refer to that file
//		for a list of all applicable permissions and restrictions.
//		
//=============================================================================
//-----------------------------------------------------------------------------
// MiscTableView.h
//
//		General-purpose 2-D display object that works with the
//		MiscTableScroll to provide row/column sizing and dragging.
//
//		This object is responsible for drawing, mouse and keyboard
//		events in the content portion of the display.
//
//-----------------------------------------------------------------------------
//-----------------------------------------------------------------------------
// $Id: MiscTableView.h,v 1.3 96/05/02 18:29:04 zarnuk Exp $
// $Log:		MiscTableView.h,v $
//	Revision 1.3  96/05/02	18:29:04  zarnuk
//	Moved keyDown: from MiscTableView to MiscTableScroll.
//	
//	Revision 1.2  96/04/23	23:19:29  zarnuk
//	Added {get/set}{First/Last}Visible{Row/Col/Slot} stuff.
//-----------------------------------------------------------------------------
extern "Objective-C" {
#import <appkit/View.h>
#import <dpsclient/dpsNeXT.h>
}
#import <misckit/MiscTableTypes.h>

@class MiscMouseTracker, MiscTableFocus, MiscTableScroll, Cell;
class MiscSparseSet;
class MiscTableBorder;

@interface MiscTableView : View
	{
	MiscTableScroll*	scroll;
	MiscTableBorder*	col_border;
	MiscTableBorder*	row_border;
	MiscBorderType		tracker_border;
	MiscMouseTracker*	tracker;
	MiscSparseSet*		oldColSel;
	MiscSparseSet*		oldRowSel;
	MiscTableFocus*		focuser;
	}

- initFrame:(NXRect const*)frameRect
		scroll:(MiscTableScroll*)i_scroll
		colInfo:(MiscTableBorder*)i_col_border
		rowInfo:(MiscTableBorder*)i_row_border;
- (void) adjustSize;

- drawCellAt: (int)row : (int)col;						// Physical coords
- drawRow: (int)row;									// Physical coord
- drawCol: (int)col;									// Physical coord

- (void) scrollCellToVisible: (int)row : (int) col;		// Physical coords
- (void) scrollRowToVisible: (int)row;					// Physical coord
- (void) scrollColToVisible: (int)col;					// Physical coord

- (int) numVisibleSlots:(MiscBorderType)b;				// all physical coords.
- (int) firstVisibleSlot:(MiscBorderType)b;
- (int) lastVisibleSlot:(MiscBorderType)b;
- (BOOL) border:(MiscBorderType)b slotIsVisible:(int)n;
- (void) border:(MiscBorderType)b setFirstVisibleSlot:(int)n;
- (void) border:(MiscBorderType)b setLastVisibleSlot:(int)n;

- (void) setSelectionMode: (MiscSelectionMode)mode;
- (void) reflectSelection;
- (void) reflectCursor;
- (void) trackBy: (MiscBorderType)b;
- (MiscBorderType) trackingBy;

- mouseDown: (NXEvent*) event;
- (void) keyboardSelect: (NXEvent const*) p;
- (void) moveFocusBy: (int) delta;
@end

#endif // __MiscTableView_h
