/*
 *  $Id: MiscLock.h,v 1.1 1997/04/07 05:43:03 croehrig Exp $
 *
 *  MiscLock  -- Foundation Kit-less NSLock, NSConditionLock
 *  Somewhat compatibile with Foundation Kit from EOF 1.1, which shipped with 
 *  NS3.3 Academic Bundle).
 *
 *  (c) 1997 Chris Roehrig <croehrig@House.ORG>
 *  All Rights Reserved
 *
 *  This notice may not be removed from this source code.
 *
 *  This object is included in the MiscKit by permission from the author
 *  and its use is governed by the MiscKit license, found in the file
 *  "License.rtf" in the MiscKit distribution.  Please refer to that file
 *  for a list of all applicable permissions and restrictions.
 *
 *  Doesn't implement timeout methods.
 *  Doesn't implement NSRecursiveLock.
 */

#import <objc/Object.h>
#import <mach/cthreads.h>

@protocol MiscLocking
- (void)lock;
- (void)unlock;
@end
 

/***************	MiscLock		***************/

@interface MiscLock:Object <MiscLocking> {
    @private
    mutex_t	mutex;
}

- (BOOL)tryLock;
    /* Returns YES iff lock acquired; returns immediately */

@end


/***************	MiscConditionLock		***************/


@interface MiscConditionLock:Object <MiscLocking> {
    @private
    mutex_t	mutex;    // lock to protect data
    volatile int	data;
    condition_t	cond;
}

- initWithCondition:(int)condition;
    // init & set condition variable
    
- (int)condition;

- (void)lockWhenCondition:(int)condition;
    // acquire lock when conditionVar == condition
    
- (BOOL)tryLock;
    /* Returns YES iff lock acquired; returns immediately */

- (BOOL)tryLockWhenCondition:(int)condition;
    // acquire lock when conditionVar == condition
    
- (void)unlockWithCondition:(int)condition;
    // release lock and update conditionVar
    
@end
