static char rcsid[] = "$Id: MiscLock.m,v 1.2 1997/04/07 06:09:09 croehrig Exp $";
/*
 *  MiscLock  -- Foundation Kit-less NSLock, NSConditionLock
 *
 *  (c) 1997 Chris Roehrig <croehrig@House.ORG>
 *  All Rights Reserved
 *
 *  This notice may not be removed from this source code.
 *
 *  This object is included in the MiscKit by permission from the author
 *  and its use is governed by the MiscKit license, found in the file
 *  "License.rtf" in the MiscKit distribution.  Please refer to that file
 *  for a list of all applicable permissions and restrictions.
 *
 */
#import <misckit/MiscLock.h>
#import <misckit/MiscThreadedApp.h>

/* VERSION = 1.1.1 */
#define MISC_LOCK_VERSION 1110

/***************	MiscLock		***************/

extern cthread_t mainThread;


@implementation MiscLock:Object


+ initialize
{
    if( [MiscLock class] == self ){
	[MiscLock setVersion:MISC_LOCK_VERSION];
    }
    return self;
}

	
- init
{
    [super init];
    mutex = mutex_alloc();
    return self;
}

- free
{
    mutex_free(mutex);
    return [super free];
}

- (void)lock
{
    if( cthread_self() != mainThread ){
	// ok to block...
	mutex_lock(mutex);
    } else {
	// Don't block main event loop; use ThreadedApp's lock
	[NXApp lock:mutex];
    }

}

- (void)unlock
{
    mutex_unlock(mutex);
    if( cthread_self() != mainThread ){
	// notify main thread that an unlock has occurred (see ThreadedApp)
	[NXApp threadDidUnlock:self];
    }
}

- (BOOL)tryLock
{
    return mutex_try_lock(mutex);
}

@end


/***************	MiscConditionLock		***************/


@implementation MiscConditionLock:Object


+ initialize
{
    if( [MiscLock class] == self ){
	[MiscLock setVersion:MISC_LOCK_VERSION];
    }
    return self;
}

	

- initWithCondition:(int)condition
{
    [super init];
    mutex = mutex_alloc();
    cond = condition_alloc();
    data = condition;

    return self;
}

- init
{
    return [self initWithCondition:0];
}

- free
{
    condition_free(cond);
    mutex_free(mutex);
    return [super free];
}

- (void)lock
{
    if( cthread_self() != mainThread ){
	// ok to block...
	mutex_lock(mutex);
    } else {
	// Don't block main event loop; use ThreadedApp's lock
	[NXApp lock:mutex];
    }
}

- (void)unlock
{
    mutex_unlock(mutex);

    // wake up other threads waiting for the lock...
    condition_broadcast(cond);

    if( cthread_self() != mainThread ){
	// notify main thread that an unlock has occurred (see ThreadedApp)
	[NXApp threadDidUnlock:self];
    }
}

- (int)condition
{
    return data;
}

- (BOOL)tryLock
{
    return mutex_try_lock(mutex);
}

- (void)lockWhenCondition:(int)condition
{
    if( cthread_self() != mainThread ){
	// blocking is ok...
	mutex_lock(mutex);
	while( data != condition )
	    condition_wait(cond,mutex);
    } else {
	// Don't block the main thread...
	[NXApp lock:mutex whenCondition:cond withData:&data is:condition];
    }
}
    

- (BOOL)tryLockWhenCondition:(int)condition
{
    if( mutex_try_lock(mutex) ){
	if( condition==data ){
	    return YES;
	} else {
	    mutex_unlock(mutex);
	    return NO;
	}
    } else {
	return NO;
    }
}
    
- (void)unlockWithCondition:(int)condition
{
    data = condition;
    mutex_unlock(mutex);

    // wake up other threads waiting for the lock...
    condition_broadcast(cond);

    // notify (wake up) the main thread 
    if( cthread_self() != mainThread ){
	[NXApp threadDidUnlock:self];
    }


}
    
@end
